# -*- coding: utf-8 -*-
#
# File: MeetingGroup.py
#
# Copyright (c) 2009 by PloneGov
# Generator: ArchGenXML Version 1.5.2
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Gaetan DELANNAY <gaetan.delannay@geezteem.com>, Gauthier BASTIEN
<gbastien@commune.sambreville.be>, Stephan GEULETTE
<stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMeeting.config import *

##code-section module-header #fill in your manual code here
from Products.PloneMeeting.utils import getCustomAdapter
from Products.PloneMeeting import PloneMeetingError
import logging
logger = logging.getLogger('PloneMeeting')
from Products.CMFCore.permissions import ModifyPortalContent
from OFS.ObjectManager import BeforeDeleteException
##/code-section module-header

schema = Schema((

    StringField(
        name='acronym',
        widget=StringWidget(
            label='Acronym',
            label_msgid='PloneMeeting_label_acronym',
            i18n_domain='PloneMeeting',
        ),
        required=True
    ),

    TextField(
        name='description',
        widget=TextAreaWidget(
            label_msgid="meetinggroup_label_description",
            label='Description',
            i18n_domain='PloneMeeting',
        ),
        accessor="Description"
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

MeetingGroup_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
from Products.PloneMeeting.model.extender import ModelExtender
MeetingGroup_schema = ModelExtender(MeetingGroup_schema, 'group').run()
##/code-section after-schema

class MeetingGroup(BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name = 'MeetingGroup'

    meta_type = 'MeetingGroup'
    portal_type = 'MeetingGroup'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 1
    #content_icon = 'MeetingGroup.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "MeetingGroup"
    typeDescMsgId = 'description_edit_meetinggroup'

    _at_rename_after_creation = True

    schema = MeetingGroup_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Manually created methods

    def getPloneGroupId(self, suffix):
        '''Returns the id of the Plone group that corresponds to me and
           p_suffix.'''
        return '%s_%s' % (self.id, suffix)

    def _createPloneGroup(self, groupSuffix):
        '''Creates the PloneGroup that corresponds to me and p_groupSuffix.'''
        groupId = self.getPloneGroupId(groupSuffix)
        enc = self.portal_properties.site_properties.getProperty(
            'default_charset')
        groupTitle = '%s (%s)' % (
            self.Title().decode(enc),
            self.utranslate(groupSuffix, domain='PloneMeeting'))
        self.portal_groups.addGroup(groupId, title=groupTitle)
        self.portal_groups.setRolesForGroup(groupId, ('MeetingObserverGlobal',))
        group = self.portal_groups.getGroupById(groupId)
        group.setProperties(meetingRole=MEETINGROLES[groupSuffix],
                            meetingGroupId=self.id)

    def getOrder(self, associatedGroupIds=None):
        '''At what position am I among all the active groups ? If
           p_associatedGroupIds is not None or empty, this method must return
           the order of the lowest group among all associated groups (me +
           associated groups).'''
        activeGroups = self.getParentNode().getActiveGroups()
        i = activeGroups.index(self)
        if associatedGroupIds:
            j = -1
            for group in activeGroups:
                j += 1
                if (group.id in associatedGroupIds) and (j < i):
                    i = j + 0.5
        return i

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        '''Creates the 3 corresponding Plone groups:
           - a group for the creators;
           - a group for the reviewers;
           - a group for the observers.'''
        # If a group with this id already exists, prevent creation from this
        # group.
        for groupSuffix in MEETINGROLES.iterkeys():
            groupId = self.getPloneGroupId(groupSuffix)
            ploneGroup = self.portal_groups.getGroupById(groupId)
            if ploneGroup:
                raise PloneMeetingError("You can't create this MeetingGroup " \
                                        "because a Plone groupe having id " \
                                        "'%s' already exists." % groupId)
        for groupSuffix in MEETINGROLES.iterkeys():
            self._createPloneGroup(groupSuffix)
        self.adapted().onEdit(isCreated=True) # Call product-specific code

    security.declarePrivate('at_post_edit_script')
    def at_post_edit_script(self): self.adapted().onEdit(isCreated=False)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        '''Checks if the current meetingGroup can be deleted:
          - it can not be linked to an existing meetingItem;
          - it can not be referenced in an existing meetingConfig;
          - the linked ploneGroups must be empty of members.'''
        # Do lighter checks first...  Check that the meetingGroup is not used
        # in a meetingConfig
        #if we are trying to remove the Plone Site, bypass this hook...
        if not item.meta_type == "Plone Site":
            for mc in self.portal_plonemeeting.objectValues('MeetingConfig'):
                # The meetingGroup can be referenced in :
                # - advices : mandatoryAdvisers and optionalAdvisers;
                # - copies : selectableCopyGroups.
                for role in MEETINGROLES.iterkeys():
                    if (self.getPloneGroupId(role) in mc.getMandatoryAdvisers()) \
                    or self.getPloneGroupId(role) in mc.getOptionalAdvisers():
                        raise BeforeDeleteException, \
                            "can_not_delete_meetinggroup_meetingconfig"
                if self.getPloneGroupId(suffix="advisers") in \
                mc.getSelectableCopyGroups():
                    raise BeforeDeleteException, \
                            "can_not_delete_meetinggroup_meetingconfig"
            # Then check that every linked Plone group is empty because we are
            # going to delete them.
            for role in MEETINGROLES.iterkeys():
                ploneGroupId = self.getPloneGroupId(role)
                group = self.portal_groups.getGroupById(ploneGroupId)
                if group and group.getMemberIds():
                    raise BeforeDeleteException, \
                        "can_not_delete_meetinggroup_plonegroup"
            # And finally check that meetingGroup is not linked to an existing item
            for brain in self.portal_catalog(meta_type="MeetingItem"):
                obj = brain.getObject()
                mgId = self.getId()
                if (obj.getProposingGroup() == mgId) or \
                   (mgId in obj.getAssociatedGroups()):
                    # The meetingGroup islinked to an existing item, we can not
                    # delete it.
                    raise BeforeDeleteException, \
                        "can_not_delete_meetinggroup_meetingitem"
            # If everything passed correctly, we delete every linked (and empty)
            # Plone groups.
            for role in MEETINGROLES.iterkeys():
                ploneGroupId = self.getPloneGroupId(role)
                group = self.portal_groups.getGroupById(ploneGroupId)
                if group:
                    self.portal_groups.removeGroup(ploneGroupId)
        BaseContent.manage_beforeDelete(self, item, container)

    security.declarePublic('getSelf')
    def getSelf(self):
        if self.__class__.__name__ != 'MeetingGroup': return self.context
        return self

    security.declarePublic('adapted')
    def adapted(self): return getCustomAdapter(self)

    security.declareProtected('Modify portal content', 'onEdit')
    def onEdit(self, isCreated): '''See doc in interfaces.py.'''



registerType(MeetingGroup, PROJECTNAME)
# end of class MeetingGroup

##code-section module-footer #fill in your manual code here
##/code-section module-footer



