## Python Script "delete_givenuid.py"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=selected_uid
##title=Deletes an object

# Get the logger
import logging
from AccessControl import Unauthorized

logger = logging.getLogger('PloneMeeting')
user = context.portal_membership.getAuthenticatedMember()

# Get the object to delete
obj = context.uid_catalog(UID=selected_uid)[0].getObject()
objectUrl = obj.absolute_url()
parent = obj.getParentNode()

# Delete the object if allowed
if obj.meta_type == 'MeetingFile':
    item = obj.getItem()
    mayDelete = True
    if item:
        mayDelete = item.wfConditions().mayDeleteAnnex(obj)
else:
    try:
        mayDelete = obj.wfConditions().mayDelete()
    except AttributeError, ae:
        mayDelete = True
if mayDelete:
    msg = 'object_deleted'
    logMsg = '"%s" object entitled "%s" was deleted by user "%s"' % \
             (obj.meta_type, obj.Title(), user.id)
    # In the case of a meeting item, delete annexes & advices, too.
    if obj.meta_type == 'MeetingItem':
        obj.removeAllAnnexes()
        obj.removeAllAdvices()
    elif obj.meta_type == 'MeetingFile':
        item = obj.getItem()
        if item:
            item.updateAnnexIndex(obj, removeAnnex=True)
    elif obj.meta_type == 'MeetingAdvice':
        item = obj.getMeetingItem()
        if item:
            item.updateAdviceIndex(obj, removeAdvice=True)

    #do a final check before calling a Manager proxy roled script
    #this is done because we want to workaround a Plone design strange
    #behaviour where a user need to have the 'Delete objects' permission
    #on the object AND on his container to be able to remove the object...
    #if we checked that we can really remove it, call a script to do the
    #work... this jut to be sure that we have "Delete objects" on the container
    if user.has_permission("Delete objects", obj):
        try:
            context.removeGivenObject(obj)
            logger.info(logMsg)
        except Exception, e:
            #catch here Exception like BeforeDeleteException
            msg = e
    else:
        raise Unauthorized
else:
    msg = 'cant_delete_object'

# Redirect the user to the correct page and display the correct message.
refererUrl = context.REQUEST['HTTP_REFERER']
if not refererUrl.startswith(objectUrl):
    urlBack = refererUrl
else:
    # We can't come back to where we came from, because it was deleted. So we
    # come back to:
    # - the PloneMeeting folder view as defined in the meeting config
    # - or the containing folder if the deleted object was an item created in
    #   portal_plonemeeting
    if 'portal_plonemeeting' in refererUrl:
        urlBack = parent.absolute_url()
    else:
        urlBack = parent.meeting_folder_view(parent)

# Remove previous message if one was there
i = urlBack.find('portal_status_message')
if i != -1:
    urlBack = urlBack[:i].strip('?&')

# Add the message to the urlBack
sep = '/?'
if urlBack.find('?') != -1:
    sep = '&'
urlBack = '%s%sportal_status_message=%s' % (urlBack, sep, msg)

return context.REQUEST.RESPONSE.redirect(urlBack)
